@@ -9,7 +9,9 @@ from typing import (Any, Dict, Iterable, List, Optional, Tuple,  # noqa: F401
                     Union, no_type_check)
 
 import neo4j
+import neobolt
 from amundsen_common.entity.resource_type import ResourceType, to_resource_type
+from amundsen_common.models.api import health_check
 from amundsen_common.models.dashboard import DashboardSummary
 from amundsen_common.models.feature import Feature, FeatureWatermark
 from amundsen_common.models.generation_code import GenerationCode
@@ -17,9 +19,9 @@ from amundsen_common.models.lineage import Lineage, LineageItem
 from amundsen_common.models.popular_table import PopularTable
 from amundsen_common.models.table import (Application, Badge, Column,
                                           ProgrammaticDescription, Reader,
-                                          Source, SqlJoin, SqlWhere, Stat,
-                                          Table, TableSummary, Tag, User,
-                                          Watermark)
+                                          ResourceReport, Source, SqlJoin,
+                                          SqlWhere, Stat, Table, TableSummary,
+                                          Tag, User, Watermark)
 from amundsen_common.models.user import User as UserEntity
 from amundsen_common.models.user import UserSchema
 from beaker.cache import CacheManager
@@ -44,12 +46,10 @@ _CACHE = CacheManager(**parse_cache_config_options({'cache.type': 'memory'}))
 # Expire cache every 11 hours + jitter
 _GET_POPULAR_RESOURCES_CACHE_EXPIRY_SEC = 11 * 60 * 60 + randint(0, 3600)
 
-
 CREATED_EPOCH_MS = 'publisher_created_epoch_ms'
 LAST_UPDATED_EPOCH_MS = 'publisher_last_updated_epoch_ms'
 PUBLISHED_TAG_PROPERTY_NAME = 'published_tag'
 
-
 LOGGER = logging.getLogger(__name__)
 
 
@@ -88,13 +88,26 @@ class Neo4jProxy(BaseProxy):
                                             encrypted=encrypted,
                                             trust=trust)  # type: Driver
 
-    def is_healthy(self) -> None:
-        # throws if cluster unhealthy or can't connect.  An alternative would be to use one of
-        # the HTTP status endpoints, which might be more specific, but don't implicitly test
-        # our configuration.
-        with self._driver.session() as session:
-            session.read_transaction(self._execute_cypher_query,
-                                     statement='CALL dbms.cluster.overview()', param_dict={})
+    def health(self) -> health_check.HealthCheck:
+        """
+        Runs one or more series of checks on the service. Can also
+        optionally return additional metadata about each check (e.g.
+        latency to database, cpu utilization, etc.).
+        """
+        checks = {}
+        try:
+            # dbms.cluster.overview() is only available for enterprise neo4j users
+            cluster_overview = self._execute_cypher_query(statement='CALL dbms.cluster.overview()', param_dict={})
+            checks = dict(cluster_overview.single())
+            checks['overview_enabled'] = True
+            status = health_check.OK
+        except neobolt.exceptions.ClientError:
+            checks = {'overview_enabled': False}
+            status = health_check.OK  # Can connect to database but plugin is not available
+        except Exception:
+            status = health_check.FAIL
+        final_checks = {f'{type(self).__name__}:connection': checks}
+        return health_check.HealthCheck(status=status, checks=final_checks)
 
     @timer_with_counter
     def get_table(self, *, table_uri: str) -> Table:
@@ -107,7 +120,7 @@ class Neo4jProxy(BaseProxy):
 
         readers = self._exec_usage_query(table_uri)
 
-        wmk_results, table_writer, timestamp_value, owners, tags, source, badges, prog_descs = \
+        wmk_results, table_writer, timestamp_value, owners, tags, source, badges, prog_descs, resource_reports = \
             self._exec_table_query(table_uri)
 
         joins, filters = self._exec_table_query_query(table_uri)
@@ -129,7 +142,8 @@ class Neo4jProxy(BaseProxy):
                       is_view=self._safe_get(last_neo4j_record, 'tbl', 'is_view'),
                       programmatic_descriptions=prog_descs,
                       common_joins=joins,
-                      common_filters=filters
+                      common_filters=filters,
+                      resource_reports=resource_reports
                       )
 
         return table
@@ -222,6 +236,7 @@ class Neo4jProxy(BaseProxy):
         OPTIONAL MATCH (tbl)-[:HAS_BADGE]->(badge:Badge)
         OPTIONAL MATCH (tbl)-[:SOURCE]->(src:Source)
         OPTIONAL MATCH (tbl)-[:DESCRIPTION]->(prog_descriptions:Programmatic_Description)
+        OPTIONAL MATCH (tbl)-[:HAS_REPORT]->(resource_reports:Report)
         RETURN collect(distinct wmk) as wmk_records,
         application,
         t.last_updated_timestamp as last_updated_timestamp,
@@ -229,7 +244,8 @@ class Neo4jProxy(BaseProxy):
         collect(distinct tag) as tag_records,
         collect(distinct badge) as badge_records,
         src,
-        collect(distinct prog_descriptions) as prog_descriptions
+        collect(distinct prog_descriptions) as prog_descriptions,
+        collect(distinct resource_reports) as resource_reports
         """)
 
         table_records = self._execute_cypher_query(statement=table_level_query,
@@ -290,7 +306,10 @@ class Neo4jProxy(BaseProxy):
             table_records.get('prog_descriptions', [])
         )
 
-        return wmk_results, table_writer, timestamp_value, owner_record, tags, src, badges, prog_descriptions
+        resource_reports = self._extract_resource_reports_from_query(table_records.get('resource_reports', []))
+
+        return wmk_results, table_writer, timestamp_value, owner_record, tags, src, badges, prog_descriptions, \
+            resource_reports
 
     @timer_with_counter
     def _exec_table_query_query(self, table_uri: str) -> Tuple:
@@ -372,6 +391,18 @@ class Neo4jProxy(BaseProxy):
         prog_descriptions.sort(key=lambda x: x.source)
         return prog_descriptions
 
+    def _extract_resource_reports_from_query(self, raw_resource_reports: dict) -> list:
+        resource_reports = []
+        for resource_report in raw_resource_reports:
+            name = resource_report.get('name')
+            if name is None:
+                LOGGER.error("A report with no name found... skipping.")
+            else:
+                resource_reports.append(ResourceReport(name=name, url=resource_report['url']))
+
+        resource_reports.sort(key=lambda x: x.name)
+        return resource_reports
+
     def _extract_joins_from_query(self, joins: List[Dict]) -> List[Dict]:
         valid_joins = []
         for join in joins:
@@ -1322,6 +1353,7 @@ class Neo4jProxy(BaseProxy):
                           last_name=record.get('last_name'),
                           full_name=record.get('full_name'),
                           is_active=record.get('is_active', True),
+                          profile_url=record.get('profile_url'),
                           github_username=record.get('github_username'),
                           team_name=record.get('team_name'),
                           slack_id=record.get('slack_id'),
